package javax.swing.text;

import javax.swing.SwingUtilities;
import java.awt.Color;
import java.awt.Shape;
import javax.swing.plaf.TextUI;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Graphics;
import java.util.Vector;

public class DefaultHighlighter extends LayeredHighlighter
{
    private static final Highlighter.Highlight[] noHighlights;
    private Vector<DefaultHighlighter.HighlightInfo> highlights;
    private JTextComponent component;
    private boolean drawsLayeredHighlights;
    private DefaultHighlighter.SafeDamager safeDamager;
    public static final LayeredHighlighter.LayerPainter DefaultPainter;
    
    public DefaultHighlighter() {
        this.highlights = new Vector();
        this.safeDamager = new DefaultHighlighter.SafeDamager();
        this.drawsLayeredHighlights = true;
    }
    
    @Override
    public void paint(Graphics graphics) {
        for (int size = this.highlights.size(), i = 0; i < size; ++i) {
            if (!(((DefaultHighlighter.HighlightInfo)this.highlights.elementAt(i)) instanceof DefaultHighlighter.LayeredHighlightInfo)) {
                Rectangle bounds = this.component.getBounds();
                Insets insets = this.component.getInsets();
                bounds.x = insets.left;
                bounds.y = insets.top;
                bounds.width -= insets.left + insets.right;
                bounds.height -= insets.top + insets.bottom;
                while (i < size) {
                    DefaultHighlighter.HighlightInfo highlightInfo = (DefaultHighlighter.HighlightInfo)this.highlights.elementAt(i);
                    if (!(highlightInfo instanceof DefaultHighlighter.LayeredHighlightInfo)) {
                        highlightInfo.getPainter().paint(graphics, highlightInfo.getStartOffset(), highlightInfo.getEndOffset(), bounds, this.component);
                    }
                    ++i;
                }
            }
        }
    }
    
    @Override
    public void install(JTextComponent component) {
        this.component = component;
        this.removeAllHighlights();
    }
    
    @Override
    public void deinstall(JTextComponent textComponent) {
        this.component = null;
    }
    
    @Override
    public Object addHighlight(int n, int n2, Highlighter.HighlightPainter painter) throws BadLocationException {
        if (n < 0) {
            throw new BadLocationException("Invalid start offset", n);
        }
        if (n2 < n) {
            throw new BadLocationException("Invalid end offset", n2);
        }
        Document document = this.component.getDocument();
        DefaultHighlighter.HighlightInfo obj = (this.getDrawsLayeredHighlights() && painter instanceof LayeredHighlighter.LayerPainter) ? new DefaultHighlighter.LayeredHighlightInfo() : new DefaultHighlighter.HighlightInfo();
        obj.painter = painter;
        obj.p0 = document.createPosition(n);
        obj.p1 = document.createPosition(n2);
        this.highlights.addElement(obj);
        this.safeDamageRange(n, n2);
        return obj;
    }
    
    @Override
    public void removeHighlight(Object obj) {
        if ((obj instanceof DefaultHighlighter.LayeredHighlightInfo)) {
            DefaultHighlighter.LayeredHighlightInfo layeredHighlightInfo = (DefaultHighlighter.LayeredHighlightInfo)obj;
            if (layeredHighlightInfo.width > 0 && layeredHighlightInfo.height > 0) {
                this.component.repaint(layeredHighlightInfo.x, layeredHighlightInfo.y, layeredHighlightInfo.width, layeredHighlightInfo.height);
            }
        }
        else {
            DefaultHighlighter.HighlightInfo highlightInfo = (DefaultHighlighter.HighlightInfo)obj;
            this.safeDamageRange(highlightInfo.p0, highlightInfo.p1);
        }
        this.highlights.removeElement(obj);
    }
    
    @Override
    public void removeAllHighlights() {
        TextUI ui = this.component.getUI();
        if (this.getDrawsLayeredHighlights()) {
            int size2 = this.highlights.size();
            if (size2 != 0) {
                int min2 = 0;
                int min3 = 0;
                int max2 = 0;
                int max3 = 0;
                int a = -1;
                int a2 = -1;
                for (int j = 0; j < size2; ++j) {
                    DefaultHighlighter.HighlightInfo highlightInfo2 = (DefaultHighlighter.HighlightInfo)this.highlights.elementAt(j);
                    if ((highlightInfo2 instanceof DefaultHighlighter.LayeredHighlightInfo)) {
                        DefaultHighlighter.LayeredHighlightInfo layeredHighlightInfo = (DefaultHighlighter.LayeredHighlightInfo)highlightInfo2;
                        min2 = Math.min(min2, layeredHighlightInfo.x);
                        min3 = Math.min(min3, layeredHighlightInfo.y);
                        max2 = Math.max(max2, layeredHighlightInfo.x + layeredHighlightInfo.width);
                        max3 = Math.max(max3, layeredHighlightInfo.y + layeredHighlightInfo.height);
                    }
                    else if (a == -1) {
                        a = highlightInfo2.p0.getOffset();
                        a2 = highlightInfo2.p1.getOffset();
                    }
                    else {
                        a = Math.min(a, highlightInfo2.p0.getOffset());
                        a2 = Math.max(a2, highlightInfo2.p1.getOffset());
                    }
                }
                if (min2 != max2 && min3 != max3) {
                    this.component.repaint(min2, min3, max2 - min2, max3 - min3);
                }
                if (a != -1) {
                    try {
                        this.safeDamageRange(a, a2);
                    }
                    catch (BadLocationException ex2) {}
                }
                this.highlights.removeAllElements();
            }
        }
        else if (ui != null) {
            int size = this.highlights.size();
            if (size != 0) {
                int min = Integer.MAX_VALUE;
                int max = 0;
                for (int i = 0; i < size; ++i) {
                    DefaultHighlighter.HighlightInfo highlightInfo = (DefaultHighlighter.HighlightInfo)this.highlights.elementAt(i);
                    min = Math.min(min, highlightInfo.p0.getOffset());
                    max = Math.max(max, highlightInfo.p1.getOffset());
                }
                try {
                    this.safeDamageRange(min, max);
                }
                catch (BadLocationException ex) {}
                this.highlights.removeAllElements();
            }
        }
    }
    
    @Override
    public void changeHighlight(Object o, int n, int n2) throws BadLocationException {
        if (n < 0) {
            throw new BadLocationException("Invalid beginning of the range", n);
        }
        if (n2 < n) {
            throw new BadLocationException("Invalid end of the range", n2);
        }
        Document document = this.component.getDocument();
        if ((o instanceof DefaultHighlighter.LayeredHighlightInfo)) {
            DefaultHighlighter.LayeredHighlightInfo layeredHighlightInfo = (DefaultHighlighter.LayeredHighlightInfo)o;
            if (layeredHighlightInfo.width > 0 && layeredHighlightInfo.height > 0) {
                this.component.repaint(layeredHighlightInfo.x, layeredHighlightInfo.y, layeredHighlightInfo.width, layeredHighlightInfo.height);
            }
            DefaultHighlighter.LayeredHighlightInfo layeredHighlightInfo2 = layeredHighlightInfo;
            int n3 = 0;
            layeredHighlightInfo.height = n3;
            layeredHighlightInfo2.width = n3;
            layeredHighlightInfo.p0 = document.createPosition(n);
            layeredHighlightInfo.p1 = document.createPosition(n2);
            this.safeDamageRange(Math.min(n, n2), Math.max(n, n2));
        }
        else {
            DefaultHighlighter.HighlightInfo highlightInfo = (DefaultHighlighter.HighlightInfo)o;
            int offset = highlightInfo.p0.getOffset();
            int offset2 = highlightInfo.p1.getOffset();
            if (n == offset) {
                this.safeDamageRange(Math.min(offset2, n2), Math.max(offset2, n2));
            }
            else if (n2 == offset2) {
                this.safeDamageRange(Math.min(n, offset), Math.max(n, offset));
            }
            else {
                this.safeDamageRange(offset, offset2);
                this.safeDamageRange(n, n2);
            }
            highlightInfo.p0 = document.createPosition(n);
            highlightInfo.p1 = document.createPosition(n2);
        }
        return;
    }
    
    @Override
    public Highlighter.Highlight[] getHighlights() {
        int size = this.highlights.size();
        if (size == 0) {
            return DefaultHighlighter.noHighlights;
        }
        Highlighter.Highlight[] anArray = new Highlighter.Highlight[size];
        this.highlights.copyInto(anArray);
        return anArray;
    }
    
    @Override
    public void paintLayeredHighlights(Graphics graphics, int n, int n2, Shape shape, JTextComponent textComponent, View view) {
        for (int i = this.highlights.size() - 1; i >= 0; --i) {
            DefaultHighlighter.HighlightInfo highlightInfo = (DefaultHighlighter.HighlightInfo)this.highlights.elementAt(i);
            if ((highlightInfo instanceof DefaultHighlighter.LayeredHighlightInfo)) {
                DefaultHighlighter.LayeredHighlightInfo layeredHighlightInfo = (DefaultHighlighter.LayeredHighlightInfo)highlightInfo;
                int startOffset = layeredHighlightInfo.getStartOffset();
                int endOffset = layeredHighlightInfo.getEndOffset();
                if ((n < startOffset && n2 > startOffset) || (n >= startOffset && n < endOffset)) {
                    layeredHighlightInfo.paintLayeredHighlights(graphics, n, n2, shape, textComponent, view);
                }
            }
        }
    }
    
    private void safeDamageRange(Position position, Position position2) {
        this.safeDamager.damageRange(position, position2);
    }
    
    private void safeDamageRange(int n, int n2) throws BadLocationException {
        Document document = this.component.getDocument();
        this.safeDamageRange(document.createPosition(n), document.createPosition(n2));
    }
    
    public void setDrawsLayeredHighlights(boolean drawsLayeredHighlights) {
        this.drawsLayeredHighlights = drawsLayeredHighlights;
    }
    
    public boolean getDrawsLayeredHighlights() {
        return this.drawsLayeredHighlights;
    }
    
    static {
        noHighlights = new Highlighter.Highlight[0];
        DefaultPainter = new DefaultHighlighter.DefaultHighlightPainter(null);
    }
    
    public static class DefaultHighlightPainter extends LayeredHighlighter.LayerPainter
    {
        private Color color;
        
        public DefaultHighlightPainter(Color color) {
            this.color = color;
        }
        
        public Color getColor() {
            return this.color;
        }
        
        @Override
        public void paint(Graphics graphics, int n, int n2, Shape shape, JTextComponent textComponent) {
            Rectangle bounds = shape.getBounds();
            try {
                TextUI ui = textComponent.getUI();
                Rectangle modelToView = ui.modelToView(textComponent, n);
                Rectangle modelToView2 = ui.modelToView(textComponent, n2);
                Color color = this.getColor();
                if (color == null) {
                    graphics.setColor(textComponent.getSelectionColor());
                }
                else {
                    graphics.setColor(color);
                }
                if (modelToView.y == modelToView2.y) {
                    Rectangle union = modelToView.union(modelToView2);
                    graphics.fillRect(union.x, union.y, union.width, union.height);
                }
                else {
                    graphics.fillRect(modelToView.x, modelToView.y, bounds.x + bounds.width - modelToView.x, modelToView.height);
                    if (modelToView.y + modelToView.height != modelToView2.y) {
                        graphics.fillRect(bounds.x, modelToView.y + modelToView.height, bounds.width, modelToView2.y - (modelToView.y + modelToView.height));
                    }
                    graphics.fillRect(bounds.x, modelToView2.y, modelToView2.x - bounds.x, modelToView2.height);
                }
            }
            catch (BadLocationException ex) {}
        }
        
        @Override
        public Shape paintLayer(Graphics graphics, int n, int n2, Shape shape, JTextComponent textComponent, View view) {
            Color color = this.getColor();
            if (color == null) {
                graphics.setColor(textComponent.getSelectionColor());
            }
            else {
                graphics.setColor(color);
            }
            Rectangle bounds;
            if (n == view.getStartOffset() && n2 == view.getEndOffset()) {
                if ((shape instanceof Rectangle)) {
                    bounds = (Rectangle)shape;
                }
                else {
                    bounds = shape.getBounds();
                }
            }
            else {
                try {
                    Shape modelToView = view.modelToView(n, Position.Bias.Forward, n2, Position.Bias.Backward, shape);
                    bounds = ((modelToView instanceof Rectangle) ? ((Rectangle)modelToView) : modelToView.getBounds());
                }
                catch (BadLocationException ex) {
                    bounds = null;
                }
            }
            if (bounds != null) {
                bounds.width = Math.max(bounds.width, 1);
                graphics.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);
                return bounds;
            }
            return bounds;
        }
    }
    
    class HighlightInfo implements Highlighter.Highlight
    {
        Position p0;
        Position p1;
        Highlighter.HighlightPainter painter;
        
        @Override
        public int getStartOffset() {
            return this.p0.getOffset();
        }
        
        @Override
        public int getEndOffset() {
            return this.p1.getOffset();
        }
        
        @Override
        public Highlighter.HighlightPainter getPainter() {
            return this.painter;
        }
    }
    
    class LayeredHighlightInfo extends DefaultHighlighter.HighlightInfo
    {
        int x;
        int y;
        int width;
        int height;
        
        void union(Shape shape) {
            if (shape == null) {
                return;
            }
            Rectangle bounds;
            if ((shape instanceof Rectangle)) {
                bounds = (Rectangle)shape;
            }
            else {
                bounds = shape.getBounds();
            }
            if (this.width == 0 || this.height == 0) {
                this.x = bounds.x;
                this.y = bounds.y;
                this.width = bounds.width;
                this.height = bounds.height;
            }
            else {
                this.width = Math.max(this.x + this.width, bounds.x + bounds.width);
                this.height = Math.max(this.y + this.height, bounds.y + bounds.height);
                this.x = Math.min(this.x, bounds.x);
                this.width -= this.x;
                this.y = Math.min(this.y, bounds.y);
                this.height -= this.y;
            }
            return;
        }
        
        void paintLayeredHighlights(Graphics graphics, int max, int min, Shape shape, JTextComponent textComponent, View view) {
            int startOffset = this.getStartOffset();
            int endOffset = this.getEndOffset();
            max = Math.max(startOffset, max);
            min = Math.min(endOffset, min);
            this.union(((LayeredHighlighter.LayerPainter)this.painter).paintLayer(graphics, max, min, shape, textComponent, view));
        }
    }
    
    class SafeDamager implements Runnable
    {
        private Vector<Position> p0;
        private Vector<Position> p1;
        private Document lastDoc;
        
        SafeDamager() {
            this.p0 = new Vector(10);
            this.p1 = new Vector(10);
            this.lastDoc = null;
        }
        
        @Override
        public synchronized void run() {
            if (DefaultHighlighter.this.component != null) {
                TextUI ui = DefaultHighlighter.this.component.getUI();
                if (ui != null && this.lastDoc == DefaultHighlighter.this.component.getDocument()) {
                    for (int size = this.p0.size(), i = 0; i < size; ++i) {
                        ui.damageRange(DefaultHighlighter.this.component, ((Position)this.p0.get(i)).getOffset(), ((Position)this.p1.get(i)).getOffset());
                    }
                }
            }
            this.p0.clear();
            this.p1.clear();
            this.lastDoc = null;
        }
        
        public synchronized void damageRange(Position e, Position e2) {
            if (DefaultHighlighter.this.component == null) {
                this.p0.clear();
                this.lastDoc = null;
                return;
            }
            int empty = this.p0.isEmpty();
            Document document = DefaultHighlighter.this.component.getDocument();
            if (document != this.lastDoc) {
                if (!this.p0.isEmpty()) {
                    this.p0.clear();
                    this.p1.clear();
                }
                this.lastDoc = document;
            }
            this.p0.add(e);
            this.p1.add(e2);
            if (!(empty == 0)) {
                SwingUtilities.invokeLater(this);
            }
            return;
        }
    }
}
